package fmon.battle

import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc}

import scalafx.Includes._
import scalafx.scene.Parent
import scalafx.scene.control._
import scalafx.scene.layout._

import fmon.battle.msg._
import fmon.stat._
import fmon.stat.Statistic._

class BattleUI extends SignalConsumer {
  @FXML var usName: jfxsc.Label = _
  @FXML var usLv: jfxsc.Label = _
  @FXML var usHealth: jfxsc.Label = _
  @FXML var usHealthBar: jfxsc.ProgressBar = _
  @FXML var themName: jfxsc.Label = _
  @FXML var themLv: jfxsc.Label = _
  @FXML var themHealth: jfxsc.Label = _
  @FXML var themHealthBar: jfxsc.ProgressBar = _
  @FXML var buttonPane: javafx.scene.layout.VBox = _
  @FXML var messages: jfxsc.TextArea = _
  
  private var engine: BattleEngine = _
  
  def setEngine(engine: BattleEngine): Unit = {
    this.engine = engine
    updateUI()
  }
  
  def updateUI(): Unit = {
    val player = engine.player.lead
    usName.text = player.name
    usLv.text = s"Lv. ${player.level}"
    usHealth.text = s"${player.hp}/${player.maxhp}"
    usHealthBar.progress = player.hp.toDouble / player.maxhp
    
    val enemy = engine.enemy.lead
    themName.text = enemy.name
    themLv.text = s"Lv. ${enemy.level}"
    themHealth.text = s"${enemy.hp}/${enemy.maxhp}"
    themHealthBar.progress = enemy.hp.toDouble / enemy.maxhp
    
    val buttons = player.base.moves.map(move => loadMoveButton(move))
    buttonPane.children = buttons
  }
  
  def onMove(move: Move): Unit = {
    val action = Action(engine.player.lead, move, engine.enemy.lead)
    engine.playTurn(action)
    updateUI()
  }
  
  def !(msg: Signal): Unit = {
    msg match {
      case Message(text) => messages.text = s"${messages.text()}${text}\n"
    }
  }
  
  private def loadMoveButton(move: Move): Button = {
    val fxmlLoader = new FXMLLoader(getClass.getResource("moveButton.fxml"))
    fxmlLoader.setBuilderFactory(new JavaFXBuilderFactory())
    val button = fxmlLoader.load[jfxsc.Button]()
    val controller = fxmlLoader.getController[MoveButton]()
    controller.setup(move, onMove)
    button
  }
}